//-----------------------------------------------------------------------------
// Source File Name: MQIsdpFrame.java
//
// Description: Contains the main method for starting the MQIsdp swing user interface
//              
// Licensed Materials - Property of IBM
//
// 5648-C63
// (C) Copyright IBM Corp. 2002, 2003 All Rights Reserved.
//
// US Government Users Restricted Rights - Use, duplication or
// disclosure restricted by GSA ADP Schedule Contract with
// IBM Corp.
//
// Version %Z% %W% %I% %E% %U%
//
//-----------------------------------------------------------------------------
package com.ibm.MQIsdp.utility;


import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;

import com.ibm.MQIsdp.*;


public class MQIsdpFrame implements ActionListener, MQIsdpSimpleCallback {
	private JPanel connPanel = new JPanel();
	private JPanel pubPanel = new JPanel();
	private JPanel subPanel = new JPanel();
	private PubPanel pubPanelContr;
	private SubPanel subPanelContr;
	private ConnOpts connDialog;
	private JComboBox ipAddress;
	private JComboBox port;
	private JButton   connect;
	private JButton   disconnect;
	private JButton   options;
	private ClientMQIsdp mqisdp = null;
	private boolean connected = false;
	private boolean traceEnabled = false;
	private static final JFrame frame = new JFrame();
	
	// Constant controlling the display of JTextFields
    public static final Dimension TEXT_FIELD_DIMENSION = new Dimension( 1000, 20 );
	// Constant controlling the display of JComboBoxes
    public static final Dimension DROP_DOWN_DIMENSION = new Dimension(35, 20);
    public static final Insets TEXT_MARGINS = new Insets(3,3,3,3);

	/**
	 * Constructor for DEDSView
	 */
	public MQIsdpFrame() {		
		super();
	}


	public static void main(String[] args) {

		final MQIsdpFrame view = new MQIsdpFrame();


        frame.setSize(375, 440);
		frame.setLocation(350, 200);
		frame.setResizable(true);

		view.init( frame.getContentPane(), false );

		frame.setVisible(true);

      
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				// Don't cleanly disconnect when the window is closed.
				// This allows things like Last Will & Testament to be easily tested by
				// closing the window to unexpectedly terminate the MQIsdp session.
				//view.disconnect();
				System.exit(0);
			}
		});
		
	}


	protected void init(Container contentPane, boolean appletFlag) {

        try {
            UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
        } catch ( Exception ex) {
        	// Don't worry if we can't set the look and feel
        }	

        setTitleText("");        

		connPanel.setLayout(new GridLayout(2, 1));
        connPanel.setBorder( new EtchedBorder() );

        // Add the panel which handles connecting and disconnecting from the broker
		contentPane.add(connPanel, BorderLayout.NORTH);
		
        // Add the panels for publish and subscribe to a JSplitPane
        // The JSplitPane allows the panels to be resized evenly, allows the user to resize the 
        // panels manually and provides a good layout for two similar panels
		JSplitPane pubsub = new JSplitPane( JSplitPane.VERTICAL_SPLIT, true,
		subPanel, pubPanel );
		pubsub.setOneTouchExpandable(true); // Allow either panel to be expanded to full size easily
		pubsub.setDividerSize(10);           // Set the divider size large enough to display the one touch expandable arrows
		pubsub.setResizeWeight(0.5);         // Resize evenly

        // Add the pubsub JSplitPane to the outer frame
		contentPane.add( pubsub, BorderLayout.CENTER );
		
		        
        ipAddress = new JComboBox();
        ipAddress.setPreferredSize( new Dimension(125,20) );
        ipAddress.setEditable( true );
        ipAddress.addItem( "127.0.0.1" );
        
        port = new JComboBox();
        port.setPreferredSize( new Dimension(65,20) );
        port.setEditable( true );
        port.addItem( "1883" );
        
        connect = new JButton( "Connect" );
        disconnect = new JButton( "Disconnect" );
        options = new JButton( "Options" );
        options.setToolTipText( "Set MQIsdp connect options" );
        connDialog = new ConnOpts( frame, "Connection options", this );
                
        connect.addActionListener(this);        
        disconnect.addActionListener(this);        
        options.addActionListener(this);        
        
        JPanel text = new JPanel();
        text.add( new JLabel("Broker TCP/IP address: ") );
        text.add( ipAddress );
        text.add( port );
        
        
        JPanel buttons = new JPanel();
        buttons.add( connect );
        buttons.add( disconnect );
        buttons.add( new JSeparator( SwingConstants.VERTICAL ) );
        buttons.add( options );
                
        connPanel.add( text );
        connPanel.add( buttons );

        pubPanelContr = new PubPanel( pubPanel, this );
        subPanelContr = new SubPanel( subPanel, this );

	}

    public void publish( String topic, byte[] message, int qos, boolean retained ) {
		setTitleText( "" );

    	if ( connected ) {
    		try {
    		  mqisdp.publish( topic, message, qos, retained );
    		} catch ( Exception ex ) {
        		setTitleText( "MQIsdp publish exception caught !" );
        		JOptionPane.showMessageDialog( frame, ex.getMessage(), "MQIsdp Publish Exception", JOptionPane.ERROR_MESSAGE );
    		}	
    	} else {
    		setTitleText( "MQIsdp client not connected !" );
    	}		
    }	
    
    public void disconnect() {
		// Disconnect from the broker
		if ( mqisdp != null ) {
  			try {
  				mqisdp.disconnect();
   			} catch ( Exception ex ) {
    			setTitleText( "MQIsdp disconnect error !" );
   				ex.printStackTrace();
   				System.exit(1);
   			}	 
   		}		
    }	
    
    public int connect( String ipAddr, int port ) throws Exception {
		// Connect to the broker
		if ( (mqisdp != null) && !ipAddr.equals(mqisdp.getServerAddress() ) ) {
			// If we have a ClientMQIsdp object and the new ip address
			// is not equal to the previous, then we need a new object
			mqisdp.terminate();
			mqisdp = null;
		}	
		if ( mqisdp == null ) {
        	mqisdp = new ClientMQIsdp( ipAddr, port );
    	    mqisdp.registerSimpleHandler( this );
    	    // Carry the trace setting over to the new ClientMQIsdp object
    	    if ( traceEnabled ) {
    	    	startTrace();
            }
		}	
        	
        if ( connDialog.isLWTTopicSet() ) {
        	// If an LWT Topic is set then do the LWT connect
        	return mqisdp.connect(connDialog.getClientID(), connDialog.isCleanSessionSelected(), connDialog.getKeepAlive(),
        	                       connDialog.getLWTTopic(), connDialog.getLWTQoS(), connDialog.getLWTData(),
        	                       connDialog.isLWTRetainSelected() );
        } else {	
        	return mqisdp.connect(connDialog.getClientID(), connDialog.isCleanSessionSelected(), connDialog.getKeepAlive() );
        }	
    }
     			
    public void subscription( String topic, int qos, boolean sub ) {
		setTitleText( "" );

    	if ( connected ) {
    		try {
    		  String[] theseTopics = new String[1];
    		  int[] theseQoS = new int[1];
    		  
    		  theseTopics[0] = topic;
    		  theseQoS[0] = qos;

              if ( sub ) {
        		  mqisdp.subscribe( theseTopics, theseQoS );
              } else {
              	  mqisdp.unsubscribe( theseTopics );
              }	  
    		  
    		} catch ( Exception ex ) {
        		setTitleText( "MQIsdp subscription exception caught !" );
        		JOptionPane.showMessageDialog( frame, ex.getMessage(), "MQIsdp Subscription Exception", JOptionPane.ERROR_MESSAGE );
    		}	
    	} else {
    		setTitleText( "MQIsdp client not connected !" );
    	}		
    }	

   	public void actionPerformed( ActionEvent e) {
   		
		setTitleText( "" );

   		if ( e.getActionCommand().equals("Connect") ) {
   			if ( connected ) {
            		setTitleText( "MQIsdp already connected !" );
   			} else {	
    			// Connect to the broker
    			try {
    				String ipAddr = (String)ipAddress.getSelectedItem();
    				String portNum = (String)port.getSelectedItem();
    				
       				if ( connect( ipAddr, Integer.parseInt( portNum ) ) == 0 ) {
       					// Successful connect. Remember the ipAddress and port in the drop downs
   	    				updateComboBoxList( ipAddress, ipAddr );
   		    			updateComboBoxList( port, portNum );
   			    		connected = true;
   				    } else {
   				    	setTitleText( "MQIsdp connect failed !" );
       				}	
  				} catch( NumberFormatException nfe ) {
   					setTitleText( "Invalid port number !" );
    			} catch ( Exception ex ) {
    				ex.printStackTrace();
            		setTitleText( "Unable to create MQIsdp connection !" );
    			}
   	 		}	 
   		} else if ( e.getActionCommand().equals("Disconnect") ) {
   			if ( connected ) {
    			// Disconnect from the broker
    			disconnect();
    			connected = false;
    			byte[] b = new byte[1];
    			b[0] = ' ';
            	subPanelContr.updateReceivedData( "", b, 0, false );
   			}	else {
    			setTitleText( "MQIsdp client not connected !" );
   			}	
   		} else {
 
//            Dimension d = frame.getSize();
//            System.out.println( "X: " + d.getWidth() + " Y:" + d.getHeight() );
            connDialog.defaultDialogSize( frame );
   			connDialog.setVisible(true);
   		}	
        		
   	}	
   	
   	// Synchronized as this may also be called on the connectionLost thread, which is
   	// created by the Java MQIsdp Client
   	public synchronized void setTitleText( String extraText ) {
   		if ( extraText.equals("") ) {
     		frame.setTitle( "MQIsdp Utility" );
   		} else {
    		frame.setTitle( "MQIsdp Utility - " + extraText );
   		}		
   	}	

    public void connectionLost() throws Exception {
    	int rc = -1;
    	setTitleText( "Connection Lost!....Reconnecting" );
    	try {
    		while ( rc == -1 ) {
        		rc = connect( mqisdp.getServerAddress(), mqisdp.getPortNumber() );
        		if ( rc == -1 ) { // Only sleep on error
            		try {
                		Thread.sleep(10000);
            		} catch (InterruptedException iex) {
        	    		// Don't care if we are interrupted
        		    }		
        		}    
    		}	
    		// Remove title text once we have reconnected
        	setTitleText( "" );
    	} catch (Exception ex) {
    		setTitleText( "MQIsdp connection broken !" );
    		disconnect();
    		throw ex;
    	}		
    }	
    
    public void publishArrived( String topic, byte[] data, int QoS, boolean retained ) {
    	subPanelContr.updateReceivedData( topic, data, QoS, retained );
    }	
    
    public void updateSubscribeTopicList( String topicName ) {
    	subPanelContr.updateTopicList( topicName );
    }	

    public void updatePublishTopicList( String topicName ) {
    	pubPanelContr.updateTopicList( topicName );
    }	
    
    // This method handles string items in JComboBox drop downs. It checks
    // to see if an item already exists in the list. If it doesn't, then it updates the list.
    // Returns true if the item was added to the list
    public boolean updateComboBoxList( JComboBox list, String itemName ) {
    	int listCount = list.getItemCount();
    	boolean found = false;
    	
    	if ( itemName == null ) {
    		return false;
    	}
    		
    	for( int i=0; i<listCount; i++ ) {
    		if ( ((String)list.getItemAt(i)).equals(itemName) ) {
    			// This topic already exists in the list, so don't add it again
    			found = true;
    			break;
    		}	
    	}	
    	
    	if ( !found ) {
    		list.addItem( itemName );
    		return true;
    	}	
    	
    	return false;
    }
    
    public void startTrace() throws Exception {
    	traceEnabled = true;
    	if ( mqisdp != null ) {
    		try {
            	mqisdp.startTrace();
    		} catch ( Exception e ) {
    			traceEnabled = false;
         		JOptionPane.showMessageDialog( frame, "MQIsdp start trace exception!",
            		 "MQIsdp Trace", JOptionPane.ERROR_MESSAGE );
    			throw e;
    		}		
    	}	
    }	

    public void stopTrace() {
    	traceEnabled = false;
    	if ( mqisdp != null ) {
        	mqisdp.stopTrace();
     		JOptionPane.showMessageDialog( frame, "Trace file mqisdp0.trc generated in the current directory",
        		 "MQIsdp Trace", JOptionPane.INFORMATION_MESSAGE );
    	}	
    }	
}