//-----------------------------------------------------------------------------
// Source File Name: SubPanel.java
//
// Description: Controls and lays out the subscription panel on the main window
//              
// Licensed Materials - Property of IBM
//
// 5648-C63
// (C) Copyright IBM Corp. 2002, 2003 All Rights Reserved.
//
// US Government Users Restricted Rights - Use, duplication or
// disclosure restricted by GSA ADP Schedule Contract with
// IBM Corp.
//
// Version %Z% %W% %I% %E% %U%
//
//-----------------------------------------------------------------------------
package com.ibm.MQIsdp.utility;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;

public class SubPanel implements ActionListener {
	private JPanel subPanel;
	private JComboBox topic;
	private JTextField receivedTopic;
	private JTextField receivedQoS;
	private JCheckBox  receivedRetain;
	private JTextArea receivedData;
	private MQIsdpFrame mqisdpMgr = null;
	private Integer[]  qos = { new Integer(0), new Integer(1), new Integer(2) };
	private JComboBox qosList;
	private boolean hexDisplay = true;
    private JLabel subLabel = null;
    private JButton hexButton = null;
    private static final String PANEL_TITLE = " Subscribe To Topics";
    
	public SubPanel( JPanel theSubPanel, MQIsdpFrame aMqisdpMgr ) {
		subPanel = theSubPanel;
        subPanel.setBorder( new EtchedBorder() );

        mqisdpMgr = aMqisdpMgr;
        
        init();
	}	
	
	public void init() {
		subPanel.setLayout( new BorderLayout() );

        // Create field to specify topics to subscribe to / unsubscribe from
        // Set a maximum size for it to stop it sizing vertically
		topic = new JComboBox();
		topic.setEditable( true );
        topic.setMaximumSize(MQIsdpFrame.TEXT_FIELD_DIMENSION);
        topic.setMaximumRowCount(5);

        // Create fields to display the topic, QoS and retained flag for which data has been received
        // Set a maximum size for it to stop it sizing vertically
		receivedTopic = new JTextField( 15 );
        receivedTopic.setMaximumSize(MQIsdpFrame.TEXT_FIELD_DIMENSION);
		receivedTopic.setEditable( false );
		receivedTopic.setBackground( Color.lightGray );
        receivedTopic.setMargin( MQIsdpFrame.TEXT_MARGINS );
        
        receivedQoS = new JTextField( 3 );
        receivedQoS.setMaximumSize( MQIsdpFrame.DROP_DOWN_DIMENSION );
        receivedQoS.setEditable( false );
        receivedQoS.setBackground( Color.lightGray );
        receivedQoS.setMargin( MQIsdpFrame.TEXT_MARGINS );

        receivedRetain = new JCheckBox();
        receivedRetain.setMaximumSize( MQIsdpFrame.DROP_DOWN_DIMENSION );
        receivedRetain.setEnabled( false );
        receivedRetain.setMargin( MQIsdpFrame.TEXT_MARGINS );
        receivedRetain.setSelected( false );

        // Create a text area to display received data in		
		receivedData = new JTextArea(3,30);	
		receivedData.setBorder( new BevelBorder( BevelBorder.LOWERED ) );
		receivedData.setEditable( false );
		receivedData.setBackground( Color.lightGray );
        receivedData.setMargin( MQIsdpFrame.TEXT_MARGINS );

		// Create a JComboBox to display the delivery QoS for a subscription
        qosList = new JComboBox( qos );
		qosList.setSelectedIndex( 0 );
		qosList.setMaximumSize( MQIsdpFrame.DROP_DOWN_DIMENSION );
		qosList.setToolTipText("The QoS at which publications will be delivered");

        // Create the components to go in the NORTH panel
        // Create a horizontal box layout in which to display the topic sent and its label
        JPanel sendTopicPanel = new JPanel();
        sendTopicPanel.setLayout( new BoxLayout( sendTopicPanel, BoxLayout.X_AXIS) );
        sendTopicPanel.add( new JLabel(" Subscribe Topic:") );
        sendTopicPanel.add( topic );
        sendTopicPanel.add( new JLabel(" Request QoS:") );
        sendTopicPanel.add( qosList );
        // Keep the qoslist from being right up against the edge of the window by adding some spaces
        sendTopicPanel.add( new JLabel("  ") );

        // Create a horizontal box layout in which to display the topic rcvd and its label
        JPanel rcvdTopicPanel = new JPanel();
        rcvdTopicPanel.setLayout( new BoxLayout( rcvdTopicPanel, BoxLayout.X_AXIS ) );
        rcvdTopicPanel.add( new JLabel(" Received Topic:") );
        rcvdTopicPanel.add( receivedTopic );
        rcvdTopicPanel.add( new JLabel(" QoS:") );
        rcvdTopicPanel.add( receivedQoS );
        rcvdTopicPanel.add( new JLabel(" Retained:") );
        rcvdTopicPanel.add( receivedRetain );
        
        // Add some white space in to stop the receivedTopic field being sized to the whole width of the GUI
        // The white space occupies the same space as the Request QoS JLabel, so the sent and rcvd topic fields line up reasonably well
//        rcvdTopicPanel.add( new JLabel("                                       ") );

        // Now to incorporate a title into the panel we place the two topic fields in a vertical
        // layout with the title "Subscribe To Topics"
        JPanel titleAndTopics = new JPanel();
        titleAndTopics.setLayout( new GridLayout( 3, 1 ) );
        
        // Add a title in BOLD        
        subLabel = new JLabel( PANEL_TITLE + " - text display");
        Font f = subLabel.getFont();
        subLabel.setFont( new Font( f.getName(), Font.BOLD, f.getSize() + 1 ) );
        
        // Add all the fields that will make up the NORTH panel
        titleAndTopics.add( subLabel );
        titleAndTopics.add( sendTopicPanel );
        titleAndTopics.add( rcvdTopicPanel );
        
        // Create the components to go in the EAST panel
        // Add the buttons to a FlowLayout to stop them resizing
		JPanel subButtonsLayout = new JPanel();
        JPanel buttons = new JPanel();
        buttons.setLayout( new GridLayout(4,1) );
        
        hexDisplay = false;
        hexButton = new JButton( "Hex" );
        hexButton.addActionListener( this );

    	JButton subButton = new JButton( "Subscribe" );
		subButton.addActionListener( this );

    	JButton unsubButton = new JButton( "Unsubscribe" );
		unsubButton.addActionListener( this );
        
        buttons.add( subButton );
        buttons.add( unsubButton );
        buttons.add( new JLabel("") );
        buttons.add( hexButton );

		subButtonsLayout.add( buttons );

        // Add the Title and Topic fields to the NORTH panel
        // Add the received publications text area to the CENTER
        // Add the Subscribe and Unsubscribe buttons to the EAST       
        subPanel.add( titleAndTopics, BorderLayout.NORTH );
        subPanel.add( new JScrollPane(receivedData), BorderLayout.CENTER );
        subPanel.add( subButtonsLayout, BorderLayout.EAST );

   	}	
		
    public boolean updateTopicList( String topicName ) {
        return mqisdpMgr.updateComboBoxList( topic, topicName );
    }

    public void actionPerformed( ActionEvent e ) {
    	String topicName = (String)topic.getSelectedItem();
    	
    	if ( updateTopicList( topicName ) ) {
    		// If we needed to update this list, then update the publisher panel's topic list
    		mqisdpMgr.updatePublishTopicList( topicName );
    	}
    	
    	if ( e.getActionCommand().equals( "Subscribe" ) ) {
            // Subscribe
            mqisdpMgr.subscription( topicName, qosList.getSelectedIndex(), true );
        } else if ( e.getActionCommand().equals("Unsubscribe") ){
            // Unsubscribe
            mqisdpMgr.subscription( topicName, 0, false );
        } else {
    		if ( hexDisplay == false ) {
        		toHexString();
    		} else {
        		toCharString();
        	}	
    	}	
    }    
    
    public void updateReceivedData( String topic, byte[] data, int QoS, boolean retained ) {
    	// Switch to a character display before adding text
    	if ( hexDisplay == true ) {
        	toCharString();
    	}	
    	receivedTopic.setText( topic );
    	receivedQoS.setText( Integer.toString(QoS) );
    	receivedRetain.setSelected( retained );
     	receivedData.setText( new String(data) );
    }	
    
    private void toHexString() {
    	String subText = receivedData.getText();
    	StringBuffer hexText = new StringBuffer();
    	
    	byte[] subBytes = subText.getBytes();

    	for( int i=0; i<subBytes.length; i++ ) {
    		int byteValue = subBytes[i];

            // Change the byte value from a signed to unsigned value
            // e.g. A byte of value 0xAA is treated as -86 and displayed incorrectly as 0xFFFFFFAA
            // Adding 256 to this value changes it to 170 which is displayed correctly as 0xAA
    		if (byteValue < 0) {
    			byteValue += 256;
    		}	

    		if ( byteValue < 16 ) {
        		hexText.append( "0" + Integer.toHexString(byteValue) );
    		} else {
        		hexText.append( Integer.toHexString(byteValue) );
    		}		
    	}	
    	
        hexDisplay = true;
        subLabel.setText( PANEL_TITLE + " - hexadecimal display" );
        mqisdpMgr.setTitleText("");
        hexButton.setText("Text");
    	receivedData.setText( hexText.toString() );
    }	
    
    private void toCharString() {
    	String hexText = receivedData.getText();
    	
    	if ( hexText.length() % 2 != 0 ) {
    		System.out.println( "Hex length" + hexText.length() );
    		mqisdpMgr.setTitleText( "Odd number of hex characters!" );
    	} else {
    		try {
                byte[] charArray = new byte[hexText.length()/2];    
        		for( int i=0; i<charArray.length; i++ ) {
                    // Take each pair of bytes from the input and convert to a character			
                    // Use the Integer parseInt method to take a 2 byte string in base 16 and turn it into an integer.
    	    		charArray[i] = (byte)Integer.parseInt( hexText.substring( i*2,(i*2)+2), 16 );
    	    		//  The code below doesn't seem to work as expected
    	    		// The parseByte method will not accept A-F chars in a base 16 pair of bytes???
    	    		//charArray[i] = Byte.parseByte( hexText.substring( i*2,(i*2)+2), 16 );
    		    }	

                hexDisplay = false;
                subLabel.setText( PANEL_TITLE + " - text display" );
    			mqisdpMgr.setTitleText( "" );
                hexButton.setText("Hex");
    		    receivedData.setText( new String(charArray) );
    		} catch( NumberFormatException nfe ) {

    			mqisdpMgr.setTitleText( "Invalid hexadecimal data!" );
    		}	    
    	}	
    }	
    
    
}
